#include "StdAfx.h"
#include "AspiForCD.h"

static const DWORD BUFFER_SIZE = 0x10000;
static const int INVALID_DEVICE_INDEX = -1;

CAspiForCD::CAspiForCD()
{
	m_canUse = false;
	m_currentDeviceIndex = INVALID_DEVICE_INDEX;

	::InitializeCriticalSection(&m_sync);
	m_hEvent = ::CreateEvent(NULL, TRUE, FALSE, NULL);

	m_pBuffer = (BYTE *)::VirtualAlloc(NULL, BUFFER_SIZE, MEM_COMMIT| MEM_RESERVE| MEM_TOP_DOWN, PAGE_READWRITE);
	::ZeroMemory(&m_status, sizeof(m_status));

	m_hWNASPI32 = ::LoadLibrary("WNASPI32");
	if (NULL == m_hWNASPI32)
		return;

	m_pGetASPI32SupportInfo = (DWORD(__cdecl *)())::GetProcAddress(m_hWNASPI32, "GetASPI32SupportInfo");
	m_pSendASPI32Command = (DWORD(__cdecl *)(LPSRB))::GetProcAddress(m_hWNASPI32, "SendASPI32Command");
	if (NULL==m_pGetASPI32SupportInfo || NULL==m_pSendASPI32Command)
		return;

	m_canUse = true;
	scanDevice();
}

CAspiForCD::~CAspiForCD()
{
	if (NULL != m_hWNASPI32)
		::FreeLibrary(m_hWNASPI32);
	::VirtualFree(m_pBuffer, 0, MEM_RELEASE);
	::CloseHandle(m_hEvent);
	::DeleteCriticalSection(&m_sync);
}

//private methods
BYTE CAspiForCD::getDeviceType(BYTE host, BYTE id)
{
	volatile SRB_GDEVBlock srb;

	::ZeroMemory((void *)&srb, sizeof(SRB_GDEVBlock));
	srb.SRB_Cmd = SC_GET_DEV_TYPE;
	srb.SRB_HaId = host;
	srb.SRB_Target = id;
	srb.SRB_DeviceType = DTYPE_UNKNOWN;

	m_pSendASPI32Command((LPSRB)&srb);

	while (SS_PENDING == srb.SRB_Status)
		::Sleep(10);

	return srb.SRB_DeviceType;
}

void CAspiForCD::getInquiryData(BYTE host, BYTE id, CAspiForCD::SDeviceInformation& data)
{
	const BYTE LENGTH = 96;
	BYTE aInquiry[] = {0x12, 0, 0, 0, LENGTH, 0};

	::ZeroMemory(m_pBuffer, LENGTH);
	sendCDB(host, id, aInquiry, 6, SRB_DIR_IN, 10);

	::ZeroMemory(&data, sizeof(SDeviceInformation));
	data.m_host = host;
	data.m_id = id;
	::lstrcpyn(data.m_aVendorID, (char *)&m_pBuffer[8], 8);
	::lstrcpyn(data.m_aProductID, (char *)&m_pBuffer[16], 16);
	::lstrcpyn(data.m_aProductRevision, (char *)&m_pBuffer[32], 4);
}

void CAspiForCD::getMaxTarget(BYTE host, BYTE& num)
{
	volatile SRB_HAInquiry srb;

	::ZeroMemory((void *)&srb, sizeof(SRB_HAInquiry));
	srb.SRB_Cmd = SC_HA_INQUIRY;
	srb.SRB_HaId = host;

	m_pSendASPI32Command((LPSRB)&srb);

	while (SS_PENDING == srb.SRB_Status)
		::Sleep(10);

	num = srb.HA_Unique[3];
}

bool CAspiForCD::sendCDB(BYTE host, BYTE id, const BYTE *pCdb, BYTE cdbLength, WORD direction, WORD timeoutTimeSeconds)
{
	SRB_ExecSCSICmd srb;
	DWORD statusAspi, statusEvent;

	::EnterCriticalSection(&m_sync);

	::ZeroMemory((void *)&srb, sizeof(SRB_ExecSCSICmd));
	srb.SRB_Cmd = SC_EXEC_SCSI_CMD;
	srb.SRB_Flags = SRB_EVENT_NOTIFY| direction;
	srb.SRB_HaId = host;
	srb.SRB_Target = id;
	srb.SRB_BufLen = BUFFER_SIZE;
	srb.SRB_BufPointer = m_pBuffer;
	srb.SRB_SenseLen = SENSE_LEN;
	srb.SRB_CDBLen = cdbLength;
	srb.SRB_PostProc = m_hEvent;
	::CopyMemory(srb.CDBByte, pCdb, cdbLength);

	::ResetEvent(m_hEvent);
	statusAspi = m_pSendASPI32Command((LPSRB)&srb);
	statusEvent = WAIT_OBJECT_0;
	if (SS_PENDING == statusAspi)
		statusEvent = ::WaitForSingleObject(m_hEvent, DWORD(timeoutTimeSeconds)*1000);

	m_status.m_status = srb.SRB_TargStat;
	m_status.m_key = srb.SenseArea[2] & 0x0f;
	m_status.m_asc = srb.SenseArea[12];
	m_status.m_ascq = srb.SenseArea[13];

	::LeaveCriticalSection(&m_sync);

	if (WAIT_OBJECT_0==statusEvent && SS_COMP==srb.SRB_Status)
		return true;

	return false;
}

bool CAspiForCD::canUse()
{
	return m_canUse;
}

BYTE *CAspiForCD::getBuffer()
{
	return m_pBuffer;
}

DWORD CAspiForCD::getBufferSize()
{
	return BUFFER_SIZE;
}

int CAspiForCD::getDeviceNumber()
{
	return m_vDeviceInformarion.size();
}

void CAspiForCD::scanDevice()
{
	BYTE host, id, hostMax, idMax;
	SDeviceInformation info;

	m_vDeviceInformarion.clear();
	hostMax = (BYTE)m_pGetASPI32SupportInfo() & 0xff;
	for (host=0; host<hostMax; host++)
	{
		getMaxTarget(host, idMax);
		for (id=0; id<idMax; id++)
		{
			if (DTYPE_CDROM != getDeviceType(host, id))
				continue;

			getInquiryData(host, id, info);
			m_vDeviceInformarion.push_back(info);
		}
	}
}

bool CAspiForCD::getDeviceInformation(int index, CAspiForCD::SDeviceInformation& info)
{
	int infoIndexMax;

	infoIndexMax = int(m_vDeviceInformarion.size());
	if (index<0 || infoIndexMax<=index)
		return false;

	info = m_vDeviceInformarion[index];

	return true;
}

bool CAspiForCD::relationDevice(int index)
{
	int infoIndexMax;

	infoIndexMax = int(m_vDeviceInformarion.size());
	if (index<0 || infoIndexMax<=index)
		return false;

	m_currentDeviceIndex = index;

	return true;
}

bool CAspiForCD::sendCDB(const BYTE *pCdb, BYTE cdbLength, WORD direction, WORD timeoutTimeSeconds)
{
	BYTE host;
	BYTE id;

	if (INVALID_DEVICE_INDEX == m_currentDeviceIndex)
		return false;

	host = m_vDeviceInformarion[m_currentDeviceIndex].m_host;
	id  = m_vDeviceInformarion[m_currentDeviceIndex].m_id;

	return sendCDB(host, id, pCdb, cdbLength, direction, timeoutTimeSeconds);
}

bool CAspiForCD::getStatus(CAspiForCD::SStatus& status)
{
	status = m_status;

	return true;
}

bool CAspiForCD::lbaToMsf(long int lba, CAspiForCD::SMSF& msf)
{
	if (-150<=lba && lba<=404849)
		;
	else if (-45150<=lba && lba<-150)
		lba += 450000;
	else
		return false;

	lba += 150;
	msf.m_minute = lba/(60*75);
	msf.m_second = (lba - (msf.m_minute*60*75))/75;
	msf.m_frame = lba-(msf.m_minute*60*75)-(msf.m_second*75);

	return true;
}

bool CAspiForCD::msfToLba(CAspiForCD::SMSF msf, long int& lba)
{
	if (99<msf.m_minute || 59<msf.m_second || 74<msf.m_frame)
		return false;

	lba = (msf.m_minute*60 + msf.m_second)*75 + msf.m_frame -150;
	if (90 <= msf.m_minute)
		lba -= 450000;

	return true;
}

bool CAspiForCD::addressToMsf(DWORD address, CAspiForCD::SMSF& msf)
{
	if ((99*60+59)*75+74 < address)
		return false;

	msf.m_minute = address/(60*75);
	msf.m_second = (address - msf.m_minute*60*75)/75;
	msf.m_frame = address - (msf.m_minute*60*75) - (msf.m_second*75);

	return true;
}

bool CAspiForCD::msfToAddress(CAspiForCD::SMSF msf, DWORD& address)
{
	if (99<msf.m_minute || 59<msf.m_second || 74<msf.m_frame)
		return false;

	if (0==msf.m_minute && msf.m_second*75+msf.m_frame<150)
		return false;

	address = (msf.m_minute*60 + msf.m_second)*75 + msf.m_frame;

	return true;
}