#include "StdAfx.h"
#include "DeviceControl.h"


CDeviceControl::CDeviceControl()
{
	m_pAspi = NULL;
}

CDeviceControl::~CDeviceControl()
{
}

bool CDeviceControl::startStopUnit(CDeviceControl::START_STOP_TYPE type, bool doImmediate)
{
	BYTE aStartStopUnit[] = {0x1b, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	aStartStopUnit[1] = BYTE(doImmediate? 1: 0);
	aStartStopUnit[4] = BYTE(type) & 0x03;

	return m_pAspi->sendCDB(aStartStopUnit, 6, SRB_DIR_SCSI, 30);
}

bool CDeviceControl::lockUnlock(CDeviceControl::LOCK_UNLOCK_TYPE type)
{
	BYTE aLockUnlock[] = {0x1e, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	aLockUnlock[4] = BYTE(type) & 0x01;

	return m_pAspi->sendCDB(aLockUnlock, 6, SRB_DIR_SCSI, 10);
}

bool CDeviceControl::connectAspi(CAspiForCD *pAspi)
{
	if (NULL == pAspi)
		return  false;

	m_pAspi = pAspi;

	return true;
}

bool CDeviceControl::disconnectAspi()
{
	m_pAspi = NULL;

	return true;
}


bool CDeviceControl::readManufacturerData(FILE *pFile)
{
	SCDRManufacturer manufacturer;
	char *p;
	char aLineBuffer[256];

	if (NULL == pFile)
		return false;

	m_vManufacturer.clear();
	while (true)
	{
		if (NULL == ::fgets(aLineBuffer, sizeof(aLineBuffer), pFile))
		{
			//if (::ferror(pFile)) //ferror̓}NɂȂĂ炵
			if (ferror(pFile))
				return false; //ǂݎ蒆ɃG[ȂAfalseԂ
			return true; //t@C̏I[܂œǂݐ؂ȂAtrueԂ
		}

		if (aLineBuffer[0] == '/') //̐擪 '/' ̏ꍇAsŜRgƂ݂Ȃ
			continue;

		if (aLineBuffer[0] == '.') //̐擪 '.' ̏ꍇAǂݎ~߂
			break;

		//񂩂 '.' ToAȍ~𐻑ЖƂē
		p = ::strchr(aLineBuffer, '.');
		if (NULL == p)
			p = "";
		else
			*::strrchr(++p, '\n') = '\0';

		manufacturer.m_strName = p;

		//ЂɑΉR[h𓾂
		::ZeroMemory(&manufacturer.m_1stCode, sizeof(CAspiForCD::SMSF));
		::ZeroMemory(&manufacturer.m_2ndCode, sizeof(CAspiForCD::SMSF));
		::sscanf(aLineBuffer, "%02d,%02d,%02d, %02d,%02d,%02d,", 
			&manufacturer.m_1stCode.m_minute,
			&manufacturer.m_1stCode.m_second,
			&manufacturer.m_1stCode.m_frame,
			&manufacturer.m_2ndCode.m_minute,
			&manufacturer.m_2ndCode.m_second,
			&manufacturer.m_2ndCode.m_frame);

		//t[̈̈ʂ0ɂ
		manufacturer.m_1stCode.m_frame /= 10;
		manufacturer.m_1stCode.m_frame *= 10;
		manufacturer.m_2ndCode.m_frame /= 10;
		manufacturer.m_2ndCode.m_frame *= 10;

		m_vManufacturer.push_back(manufacturer);
	}

	return true;
}

bool CDeviceControl::testUnitReady(CAspiForCD::SStatus& status)
{
	BYTE aTestUnitReady[] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	if (!m_pAspi->sendCDB(aTestUnitReady, 6))
		return false;

	status.m_status = m_pAspi->getStatus(status);

	return true;
}

bool CDeviceControl::waitForReady()
{
	CAspiForCD::SStatus status;

	while (true)
	{
		if (!testUnitReady(status))
			return false;

		if (KEY_NOTREADY != status.m_status)
			break;
	}

	return true;
}

bool CDeviceControl::getDiskStatus(CDeviceControl::DISK_STATUS& status)
{
	BYTE *p;
	const BYTE LENGTH = 0x20;
	BYTE aReadDiscInformation[] = {0x51, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, LENGTH, 0x00};

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, LENGTH);

	if (!m_pAspi->sendCDB(aReadDiscInformation, 10, SRB_DIR_IN, 10))
		return false;

	switch (p[2] & 0x03)
	{
	case 0:
		status = DISK_STATUS_EMPTY;
		break;

	case 1:
		status = DISK_STATUS_INCOMLETE;
		break;

	case 2:
		status = DISK_STATUS_COMPLETE;
		break;

	case 3:
		status = DISK_STATUS_OTHORS;
		break;
	}

	return true;
}

bool CDeviceControl::modeSense(BYTE pageCode, BYTE *pPage, BYTE pageLen, CDeviceControl::PAGE_CONTROL pageControl, WORD *pPageLenFromResult)
{
	WORD pageLenFromResult;
	WORD pageIndex;
	BYTE *pBuf;
	BYTE aModeSense6[] = {0x1a, 0x00, 0x00, 0x00, 0x00, 0x00};
	BYTE aModeSense10[] = {0x5a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	if (NULL == pPage || pageCode == 0x3f)
		return false;

	pBuf = m_pAspi->getBuffer();
	::ZeroMemory(pBuf, pageLen);
	aModeSense6[2] = aModeSense10[2] = (BYTE(pageControl)<<6)| (pageCode&0x3f);
	aModeSense6[4] = aModeSense10[8] = pageLen;

	if (m_pAspi->sendCDB(aModeSense6, 6, SRB_DIR_IN, 10)) 
	{
		pageIndex = 4+pBuf[3];
		pageLenFromResult = pBuf[pageIndex+1] + 2;

		if (NULL != pPageLenFromResult)
			*pPageLenFromResult = pageLenFromResult;

		if (pageLenFromResult < pageLen)
			pageLen = pageLenFromResult;
		::CopyMemory(pPage, pBuf+pageIndex, pageLen);
		return true;
	}
	else if (m_pAspi->sendCDB(aModeSense10, 10, SRB_DIR_IN, 10))
	{
		pageIndex = 8+pBuf[6]*256+pBuf[7];
		pageLenFromResult = pBuf[pageIndex+1] + 2;

		if (NULL != pPageLenFromResult)
			*pPageLenFromResult = pageLenFromResult;

		if (pageLenFromResult < pageLen)
			pageLen = pageLenFromResult;
		::CopyMemory(pPage, pBuf+pageIndex, pageLen);
		return true;
	}
	else
		return false;
}

bool CDeviceControl::getManufacturerName(CAspiForCD::SMSF leadIn, std::string& name)
{
	SCDRManufacturer manufacturer;
	std::vector<SCDRManufacturer>::iterator it;

	//manufacture.txt  format ł́AȃR[h 00,00,00 ƂĂ̂ŁA
	//Yꍇ͐Ж𓾂ȂƂB
	if (0 == leadIn.m_minute)
		return false;

	leadIn.m_frame /= 10;
	leadIn.m_frame *= 10;

	it = m_vManufacturer.begin();
	while (it != m_vManufacturer.end())
	{
		manufacturer = (*it++);
		if (leadIn==manufacturer.m_1stCode || leadIn==manufacturer.m_2ndCode)
		{
			name = manufacturer.m_strName;
			return true;
		}
	}

	return false;
}

bool CDeviceControl::getATIP(CDeviceControl::SATIP& atip)
{
	BYTE *p;
	const BYTE LENGTH = 0x20;
	BYTE aReadATIP[] = {0x43, 0x02, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, LENGTH, 0x00};
	CAspiForCD::SStatus status;

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, LENGTH);

	if (!m_pAspi->sendCDB(aReadATIP, 10, SRB_DIR_IN, 10))
		return false;

	m_pAspi->getStatus(status);
	if (STATUS_GOOD != status.m_status)
		return false;

	atip.m_diskType = (p[6]>>6)&1;
	atip.m_diskSubType = (p[6]>>3)&7;
	atip.m_msfLeadin.m_minute = p[8];
	atip.m_msfLeadin.m_second = p[9];
	atip.m_msfLeadin.m_frame = p[10];
	atip.m_msfLeadout.m_minute = p[12];
	atip.m_msfLeadout.m_second = p[13];
	atip.m_msfLeadout.m_frame = p[14];

	return true;
}

bool CDeviceControl::getCapacity(long int& capacity)
{
	BYTE *p;
	const BYTE LENGTH = 8;
	BYTE aReadCapacity[] = {0x25, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, LENGTH);

	if (!m_pAspi->sendCDB(aReadCapacity, 10, SRB_DIR_IN, 10))
		return false;

	capacity = (DWORD(p[0])<<24) | (DWORD(p[1])<<16) | (DWORD(p[2])<<8) | DWORD(p[3]);

	return true;
}

bool CDeviceControl::getLastTrack(BYTE& track)
{
	BYTE *p;
	const BYTE LENGTH = 0x20;
	BYTE aReadDiscInformation[] = {0x51, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, LENGTH, 0x00};

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, LENGTH);

	if (!m_pAspi->sendCDB(aReadDiscInformation, 10, SRB_DIR_IN, 10))
		return false;

	track = p[6];

	return true;
}

bool CDeviceControl::getTrackPosition(BYTE track, long int& start, long int& end, bool& isValid)
{
	BYTE *p;
	const BYTE LENGTH = 36;
	BYTE aReadTrackInformation[] = {0x52, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, LENGTH, 0x00};
	long int tmp;

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, LENGTH);

	aReadTrackInformation[5] = track;
	if (!m_pAspi->sendCDB(aReadTrackInformation, 10, SRB_DIR_IN, 10))
		return false;

	start = (DWORD(p[8])<<24) | (DWORD(p[9])<<16) | (DWORD(p[10])<<8) | DWORD(p[11]);
	tmp = (DWORD(p[24])<<24) | (DWORD(p[25])<<16) | (DWORD(p[26])<<8) | DWORD(p[27]);
	end = start + tmp;

	if (p[6] & 0x40) //is BLANK track ?
		isValid = false;
	else
		isValid = true;

	return true;
}

bool CDeviceControl::getTOC(CDeviceControl::READ_TOC_TYPE type, BYTE parameter)
{
	BYTE *p;
	BYTE aReadToc[] = {0x43, 0x00, BYTE(type), 0x00, 0x00, 0x00, parameter, 0xff, 0xff, 0x00};

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, m_pAspi->getBufferSize());

	return m_pAspi->sendCDB(aReadToc, 10, SRB_DIR_IN);
}

bool CDeviceControl::setSpeed(WORD r, WORD w)
{
	BYTE aSetCDSpeed[] = {0xbb, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	aSetCDSpeed[2] = (r>>8)&0xff;
	aSetCDSpeed[3] = r&0xff;
	aSetCDSpeed[4] = (w>>8)&0xff;
	aSetCDSpeed[5] = w&0xff;

	return m_pAspi->sendCDB(aSetCDSpeed, 12);
}

bool CDeviceControl::getSpeed(WORD& r, WORD& w, WORD& readMax, CDeviceControl::ROTATION_TYPE& t)
{
	BYTE aPage[30];

	if (NULL == m_pAspi)
		return false;

	::ZeroMemory(aPage, sizeof(aPage));
	if (!modeSense(0x2a, aPage, sizeof(aPage)))
		return false;

	r = aPage[14]*256 + aPage[15];
	w = aPage[28]*256 + aPage[29];
	readMax = aPage[8]*256 + aPage[9];
	switch (aPage[27] & 3)
	{
	case 0:
		t = ROTATION_TYPE_CLV;
		break;

	case 1:
		t = ROTATION_TYPE_CAV_PURE;
		break;

	case 2:
		t = ROTATION_TYPE_RESERVE_1;
		break;

	case 3:
		t = ROTATION_TYPE_RESERVE_2;
		break;
	}

	return true;
}

bool CDeviceControl::load(bool doImmediate)
{
	return startStopUnit(START_STOP_LOAD, doImmediate);
}

bool CDeviceControl::unload(bool doImmediate)
{
	return startStopUnit(START_STOP_UNLOAD, doImmediate);
}

bool CDeviceControl::rotate(bool doImmediate)
{
	return startStopUnit(START_STOP_ROTATE, doImmediate);
}

bool CDeviceControl::stop(bool doImmediate)
{
	return startStopUnit(START_STOP_STOP, doImmediate);
}

bool CDeviceControl::lock()
{
	return lockUnlock(LOCK_UNLOCK_LOCK);
}

bool CDeviceControl::unlock()
{
	return lockUnlock(LOCK_UNLOCK_UNLOCK);
}


bool CDeviceControl::readCD(long int lba, WORD blocks, BYTE where, BYTE subChannel, WORD timeoutTimeSeconds)
{
	DWORD address;
	BYTE *p;
	BYTE aReadCD[] = {0xbe, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};

	if (NULL == m_pAspi)
		return false;

	p = m_pAspi->getBuffer();
	::ZeroMemory(p, m_pAspi->getBufferSize());

	address = DWORD(lba);
	aReadCD[2] = BYTE(address>>24);
	aReadCD[3] = BYTE(address>>16);
	aReadCD[4] = BYTE(address>> 8);
	aReadCD[5] = BYTE(address>> 0);

	aReadCD[6] = BYTE(0);
	aReadCD[7] = BYTE(blocks>>8);
	aReadCD[8] = BYTE(blocks>>0);

	aReadCD[9] = where;

	aReadCD[10] = subChannel&0x07;

	return m_pAspi->sendCDB(aReadCD, 10, SRB_DIR_IN, timeoutTimeSeconds);
}