#ifndef __DEVICECONTROL_H__
#define __DEVICECONTROL_H__

#include <stdio.h>
#include <string>
#include <vector>
#include "AspiForCD.h"

class CDeviceControl
{
public:
	struct SATIP
	{
		BYTE m_diskType;
		BYTE m_diskSubType;
		CAspiForCD::SMSF m_msfLeadin;
		CAspiForCD::SMSF m_msfLeadout;
	};

	struct SCDRManufacturer
	{
		CAspiForCD::SMSF m_1stCode;
		CAspiForCD::SMSF m_2ndCode;
		std::string m_strName;
	};

	enum DISK_STATUS
	{
		DISK_STATUS_EMPTY,
		DISK_STATUS_INCOMLETE,
		DISK_STATUS_COMPLETE,
		DISK_STATUS_OTHORS,
	};

	enum ROTATION_TYPE
	{
		ROTATION_TYPE_CLV,
		ROTATION_TYPE_CAV_PURE,
		ROTATION_TYPE_RESERVE_1,
		ROTATION_TYPE_RESERVE_2,
	};

	enum PAGE_CONTROL
	{
		PAGE_CONTROL_CURRENT,
		PAGE_CONTROL_CHANGEABLE,
		PAGE_CONTROL_DEFAULT,
		PAGE_CONTROL_SAVED,
	};

	enum READ_TOC_TYPE
	{
		READ_TOC_TYPE_TOC,
		READ_TOC_TYPE_SESSION_INFORMATION,
		READ_TOC_TYPE_FULL_TOC,
		READ_TOC_TYPE_PMA,
		READ_TOC_TYPE_ATIP,
		READ_TOC_TYPE_CDTEXT,
	};

protected:
	enum START_STOP_TYPE
	{
		START_STOP_STOP,
		START_STOP_ROTATE,
		START_STOP_UNLOAD,
		START_STOP_LOAD,
	};
	enum LOCK_UNLOCK_TYPE
	{
		LOCK_UNLOCK_UNLOCK,
		LOCK_UNLOCK_LOCK,
	};

	CAspiForCD *m_pAspi;
	std::vector<SCDRManufacturer> m_vManufacturer;

	bool startStopUnit(CDeviceControl::START_STOP_TYPE type, bool doImmediate=false);
	bool lockUnlock(CDeviceControl::LOCK_UNLOCK_TYPE type);

public:
	CDeviceControl();
	virtual ~CDeviceControl();

	bool connectAspi(CAspiForCD *pAspi);
	bool disconnectAspi();

	bool readManufacturerData(FILE *pFile);

	bool testUnitReady(CAspiForCD::SStatus& status);
	bool waitForReady();
	bool getDiskStatus(CDeviceControl::DISK_STATUS& status);
	bool modeSense(BYTE pageCode, BYTE *pPage, BYTE pageLen, CDeviceControl::PAGE_CONTROL pageControl=PAGE_CONTROL_CURRENT, WORD *pPageLenFromResult=NULL);

	bool getManufacturerName(CAspiForCD::SMSF leadIn, std::string& name);
	bool getATIP(CDeviceControl::SATIP& atip);
	bool getCapacity(long int& capacity);
	bool getLastTrack(BYTE& track);
	bool getTrackPosition(BYTE track, long int& start, long int& end, bool& isValid);
	bool getTOC(CDeviceControl::READ_TOC_TYPE type, BYTE parameter);

	bool setSpeed(WORD r, WORD w);
	bool getSpeed(WORD& r, WORD& w, WORD& readMax, CDeviceControl::ROTATION_TYPE& t);

	bool load(bool doImmediate=false);
	bool unload(bool doImmediate=false);
	bool rotate(bool doImmediate=false);
	bool stop(bool doImmediate=false);

	bool lock();
	bool unlock();

	bool readCD(long int lba, WORD blocks, BYTE where=0x10, BYTE subChannel=0, WORD timeoutTimeSeconds=10);
};

#endif //__DEVICECONTROL_H__